//
//  DRFileNotificationSubscription.m
//  MoreWaysToSkinADock
//
//  Created by David Remahl on Fri Apr 12 2002.
//  Copyright (c) 2001 Infinity-to-the-Power-of-Infinity. All rights reserved.
//

#import "DRFileNotificationSubscription.h"


@implementation DRFileNotificationSubscription

- (id)initForDirectory:(NSString *)path target:(id)newTarget action:(SEL)newAction
{
    self = [super init];

    if(self != nil)
    {
		FSRef dirRef;
		BOOL isDir;
		OSStatus err;

        if([[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isDir] && isDir)
        {
			CFURLRef urlRef;
			urlRef = CFURLCreateWithFileSystemPath(NULL, (CFStringRef)path, kCFURLPOSIXPathStyle, NO);
			if(CFURLGetFSRef(urlRef, &dirRef))
			{
				subUPP = NewFNSubscriptionUPP((FNSubscriptionProcPtr)DRFileSubscriptionProc);

				[self setTarget:newTarget];
				[self setAction:newAction];

				err = FNSubscribe( &dirRef, subUPP, (void*)self, 0, &subRef );
				CFRelease(urlRef);
				if(err == noErr)
				{
					watchedPath = [path copy];
				}
				else
				{
					[self release];
					self = nil;
					CFRelease(urlRef);
					[[NSException exceptionWithName:@"DRFileNotificationSubscriptionException" reason:@"could not subscribe" userInfo:nil] raise];
				}
			}
			else
			{
				CFRelease(urlRef);
				[self release];
				self = nil;
				[[NSException exceptionWithName:NSInvalidArgumentException reason:@"could not get FSRef for specified path" userInfo:nil] raise];
			}
		}
        else
        {
            [self release];
			self = nil;
            [[NSException exceptionWithName:NSInvalidArgumentException reason:@"Filesystem notification requested for non-directory file" userInfo:nil] raise];
        }   
    }

    return self;
}

- (void)dealloc
{
	if(subUPP != nil)
	{
		DisposeFNSubscriptionUPP(subUPP);
	}
	if(subRef != nil)
	{
		FNUnsubscribe(subRef);
	}
	[watchedPath release];
    [super dealloc];
}

- (void)setTarget:(id)newTarget
{
    target = newTarget;
}

- (void)setAction:(SEL)newAction
{
    action = newAction;
}

- (id)target
{
    return target;
}

- (SEL)action
{
    return action;
}

- (NSString*)watchedPath
{
    return watchedPath;
}

- (void)fire
{
    [target performSelector:action withObject:self];
}

@end

void DRFileSubscriptionProc(FNMessage message, OptionBits flags, DRFileNotificationSubscription* refcon, FNSubscriptionRef subscription)
{
    [refcon fire];
}
