//
//  ZBindingManager.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/10/10.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZBindingManager.h"


@implementation ZBindingManager
+ (ZBindingManager *)managerForObject:(id)object keyPath:(NSString *)keyPath
{
	return [[[ZBindingManager alloc] initForObject:object keyPath:keyPath] autorelease];
}

- (id)initForObject:(id)object keyPath:(NSString *)keypath
{
	self = [super init];
	if(self != nil)
	{
		_object = object; // not retaind. 
		_keyPath = [keypath retain];
	}
	return self;
}

- (void)dealloc
{
	[self unbind];
	[_keyPath release];
	[super dealloc];
}

NSString *targetObservationContext = @"targetObservationContext";
NSString *objectObservationContext = @"objectObservationContext";

- (void)bindToObject:(id)bindingTarget
		withKeyPath:(NSString *)keyPath
		options:(NSDictionary *)options
{
	[self unbind];
	
	_bindingTarget = [bindingTarget retain];
	_bindingTargetKeyPath = [keyPath retain];
	_bindingOptions = [options retain];
	
	[_object addObserver:self forKeyPath:_keyPath options:0 context:objectObservationContext];
	[_bindingTarget addObserver:self forKeyPath:_bindingTargetKeyPath options:NSKeyValueObservingOptionInitial context:targetObservationContext];
}

- (void)unbind
{
	if(_bindingTarget != nil)
	{
		[_bindingTarget removeObserver:self forKeyPath:_bindingTargetKeyPath];
		[_bindingTarget release];
		_bindingTarget = nil;
		[_bindingTargetKeyPath release];
		_bindingTargetKeyPath = nil;
		[_bindingOptions release];
		_bindingOptions = nil;
		[_object removeObserver:self forKeyPath:_keyPath];
	}
}

- (NSValueTransformer *)valueTransformer
{
	NSValueTransformer *vt = nil;
	if(_bindingOptions != nil)
	{
		vt = [_bindingOptions objectForKey:NSValueTransformerBindingOption];
		if(vt == nil)
		{
			NSString *name = [_bindingOptions objectForKey:NSValueTransformerNameBindingOption];
			if(name != nil)
			{
				vt = [NSValueTransformer valueTransformerForName:name];
			}
		}
	}
	return vt;
}

- (void)observeValueForKeyPath:(NSString *)keyPath
		ofObject:(id)object
		change:(NSDictionary *)change
		context:(void *)context
{
	if(context == targetObservationContext)
	{
		id value = [object valueForKeyPath:keyPath];
		NSValueTransformer *vt = [self valueTransformer];
		if(vt != nil)
		{
			value = [vt transformedValue:value];
		}
		[_object removeObserver:self forKeyPath:_keyPath];
		[_object setValue:value forKeyPath:_keyPath];
		[_object addObserver:self forKeyPath:_keyPath options:0 context:objectObservationContext];
	}
	else if(context == objectObservationContext)
	{
		id value = [object valueForKeyPath:keyPath];
		NSValueTransformer *vt = [self valueTransformer];
		if(vt != nil)
		{
			value = [vt reverseTransformedValue:value];
		}
		[_bindingTarget removeObserver:self forKeyPath:_bindingTargetKeyPath];
		[_bindingTarget setValue:value forKeyPath:_bindingTargetKeyPath];
		[_bindingTarget addObserver:self forKeyPath:_bindingTargetKeyPath options:0 context:targetObservationContext];
	}
}
@end
