//
//  ZColorAction.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/10/09.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZColorAction.h"

#import "ZParseTree.h"
#import "ZSyntax.h"

NSColor *readColorString(NSString *colorString)
{
	NSString *rs = [colorString substringWithRange:NSMakeRange(1, 2)];
	NSString *gs = [colorString substringWithRange:NSMakeRange(3, 2)];
	NSString *bs = [colorString substringWithRange:NSMakeRange(5, 2)];
	
	unsigned int r, g, b;
	
	if([[NSScanner scannerWithString:rs] scanHexInt:&r] &&
	   [[NSScanner scannerWithString:gs] scanHexInt:&g] &&
	   [[NSScanner scannerWithString:bs] scanHexInt:&b])
	{
		return [NSColor colorWithCalibratedRed:r/256.0 green:g/256.0 blue:b/256.0 alpha:1];
	}
	else
	{
		return nil;
	}
}

@implementation ZColorAction

+ (ZColorAction *)actionWithColorString:(NSString *)colorString bgColorString:(NSString *)bgColorString
{
	return [[[ZColorAction alloc] initWithColorString:colorString bgColorString:bgColorString] autorelease];
}

+ (ZColorAction *)actionWithColorString:(NSString *)colorString
{
	return [[[ZColorAction alloc] initWithColorString:colorString bgColorString:nil] autorelease];
}

- (id)initWithColorString:(NSString *)colorString bgColorString:(NSString *)bgColorString
{
	self = [super init];
	if(self != nil)
	{
		@try
		{
			if(colorString != nil)
			{
				if([colorString hasPrefix:@"#"])
				{
					_color = [readColorString(colorString) retain];
				}
				else
				{
					_colorName = [colorString retain];
				}
			}
			
			if(bgColorString != nil)
			{
				if([bgColorString hasPrefix:@"#"])
				{
					_bgColor = [readColorString(bgColorString) retain];
				}
				else
				{
					_bgColorName = [bgColorString retain];
				}
			}
		}
		@catch(NSException *e)
		{
			[self release];
			@throw;
		}
	}
	return self;
}

- (id)initWithColorString:(NSString *)colorString
{
	self = [super init];
	if(self != nil)
	{
		if([colorString hasPrefix:@"#"])
		{
			@try
			{
				NSString *rs = [colorString substringWithRange:NSMakeRange(1, 2)];
				NSString *gs = [colorString substringWithRange:NSMakeRange(3, 2)];
				NSString *bs = [colorString substringWithRange:NSMakeRange(5, 2)];
				
				unsigned int r, g, b;
				
				if([[NSScanner scannerWithString:rs] scanHexInt:&r] &&
				   [[NSScanner scannerWithString:gs] scanHexInt:&g] &&
				   [[NSScanner scannerWithString:bs] scanHexInt:&b])
				{
					_color = [NSColor colorWithCalibratedRed:r/256.0 green:g/256.0 blue:b/256.0 alpha:1];
					[_color retain];
				}
				else
				{
					[NSException raise:@"ColorActionFormatException" format:@"color format must be specified by #rrggbb in hex."];
				}
			}
			@catch(NSException *e)
			{
				[self release];
				return nil;
			}
		}
		else
		{
			_colorName = [colorString retain];
		}
	}
	return self;
}

- (void)dealloc
{
	[_colorName release];
	[_color release];
	[super dealloc];
}

- (void)runWithTree:(ZParseTree *)tree range:(NSRange)range context:(void *)context
{
	NSDictionary *args = context;
	
	NSDictionary *colors = [args valueForKey:ActionContextColorsKey];
	NSMutableAttributedString *as = [args valueForKey:ActionContextStringKey];
	
	NSColor *color = _color;
	if(color == nil && _colorName != nil)
	{
		color = [colors valueForKey:_colorName];
	}
	
	NSColor *bgColor = _bgColor;
	if(bgColor == nil && _bgColorName != nil)
	{
		bgColor = [colors valueForKey:_bgColorName];
	}
	
	if(color != nil)
	{
		[as addAttribute:NSForegroundColorAttributeName value:color range:range];
	}
	if(bgColor != nil)
	{
		[as addAttribute:NSBackgroundColorAttributeName value:bgColor range:range];
	}
}
@end
