//
//  ZColoringParser.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/10/09.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZColoringParser.h"

#import "ZKeywordParser.h"
#import "ZTokenParser.h"
#import "ZColorAction.h"
#import "ZSyntax.h"
#import "ZParenthesisAction.h"

NSString *StateParserBeginKey = @"begin";
NSString *StateParserEndKey = @"end";
NSString *StateParserTokenKey = @"token";
NSString *StateParserSubstatesKey = @"substates";
NSString *StateParserKeywordsKey = @"keywords";
NSString *StateParserParenthesesKey = @"parentheses";
NSString *StateParserColorKey = @"color";
NSString *StateParserBackgroundKey = @"background";
NSString *KeywordRegexsKey = @"regexes";
NSString *ParenthesisBeginKey = @"begin";
NSString *ParenthesisEndKey = @"end";

@implementation ZColoringParser
+ (void)addKeywrodsTo:(NSMutableArray *)to desc:(NSDictionary *)desc
{
	NSMutableArray *parsers = [NSMutableArray array];
	NSArray *keywords = [desc objectForKey:StateParserKeywordsKey];
	if(keywords != nil)
	{
		for(NSDictionary *d in keywords)
		{
			NSString *color = [d objectForKey:StateParserColorKey];
			NSString *bgColor = [d objectForKey:StateParserBackgroundKey];
			NSArray *regexes = [d objectForKey:KeywordRegexsKey];
			ZColorAction *action = nil;
			if(color != nil || bgColor != nil)
			{
				action = [ZColorAction actionWithColorString:color bgColorString:bgColor];
			}
			
			if(regexes != nil)
			{
				for(NSString *r in regexes)
				{
					ZKeywordParser *parser = [ZKeywordParser parserWithRegex:r];
					if(action != nil)
					{
						[[parser actions] addObject:action];
					}
					[parsers addObject:parser];
				}
			}
		}
	}
	
	NSString *token = [desc objectForKey:StateParserTokenKey];
	if(token != nil)
	{
		ZTokenParser *tokenParser = [ZTokenParser parserWithToken:token keywordParsers:parsers];
		[to addObject:tokenParser];
	}
	else
	{
		[to addObjectsFromArray:parsers];
	}
}

+ (void)addSubstatesTo:(NSMutableArray *)to desc:(NSDictionary *)desc
{
	NSArray *substates = [desc objectForKey:StateParserSubstatesKey];
	if(substates != nil)
	{
		for(NSDictionary *d in substates)
		{
			ZColoringParser *state = [ZColoringParser parserWithDescription:d topLevel:NO];
			[to addObject:state];
		}
	}
}

+ (void)addParensTo:(NSMutableArray *)to desc:(NSDictionary *)desc
{
	NSArray *parens = [desc objectForKey:StateParserParenthesesKey];
	if(parens != nil)
	{
		for(NSDictionary *d in parens)
		{
			ZKeywordParser *begin = [ZKeywordParser parserWithRegex:[d objectForKey:ParenthesisBeginKey]];
			ZParenthesisAction *beginAction = [ZParenthesisAction action];
			[beginAction setIsEnd:NO];
			
			ZKeywordParser *end = [ZKeywordParser parserWithRegex:[d objectForKey:ParenthesisEndKey]];
			ZParenthesisAction *endAction = [ZParenthesisAction action];
			[endAction setIsEnd:YES];
			
			[beginAction setCorrespond:endAction];
			[endAction setCorrespond:beginAction];
			
			[[begin actions] addObject:beginAction];
			[[end actions] addObject:endAction];
			
			NSString *color = [d objectForKey:StateParserColorKey];
			NSString *bgColor = [d objectForKey:StateParserBackgroundKey];
			if(color == nil)
			{
				color = ColorsParenthesisKey;
			}
			ZColorAction *c = [ZColorAction actionWithColorString:color bgColorString:bgColor];
			[[begin actions] addObject:c];
			[[end actions] addObject:c];
			
			[to addObject:begin];
			[to addObject:end];
		}
	}
}

+ (ZColoringParser *)parserWithDescription:(NSDictionary *)description topLevel:(BOOL)topLevel
{
	NSString *begin = [description objectForKey:StateParserBeginKey];
	NSString *end = [description objectForKey:StateParserEndKey];
		
	NSString *color = [description objectForKey:StateParserColorKey];
	NSString *bgColor = [description objectForKey:StateParserBackgroundKey];
	
	NSMutableArray *subparsers = [NSMutableArray array];
	[self addSubstatesTo:subparsers desc:description];
	[self addParensTo:subparsers desc:description];
	[self addKeywrodsTo:subparsers desc:description];
	
	ZColoringParser *parser = [[[ZColoringParser alloc] initWithBegin:begin end:end subparsers:subparsers] autorelease];
	if(topLevel)
	{
		if(color == nil)
		{
			color = ColorsTextKey;
		}
		if(bgColor == nil)
		{
			bgColor = ColorsBackgroundKey;
		}
	}
	if(color != nil || bgColor != nil)
	{
		[[parser actions] addObject:[ZColorAction actionWithColorString:color bgColorString:bgColor]];
	}
	
	return parser;
}
@end
