//
//  ZEditFileSystemItem.m
//  ZEdit
//
//  Created by 増田 勇樹 on 07/03/11.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "ZEditFileSystemItem.h"

#import "ZEditDocument.h"
#import "ZEditTextDocument.h"
#import "ZEditDirectoryDocument.h"
#import "ZEditWindowController.h"

@implementation ZEditFileSystemItem
- (void)dealloc
{
	[self setDocument:nil];
	[super dealloc];
}

- (void)setSharedWindowController:(ZEditWindowController *)windowController
{
	_sharedWindowController = windowController;
}

- (ZEditWindowController *)sharedWindowController
{
	if(_sharedWindowController != nil)
	{
		return _sharedWindowController;
	}
	else
	{
		if([self parent] != nil)
		{
			return [(ZEditFileSystemItem *)[self parent] sharedWindowController];
		}
		else if(_owner != nil)
		{
			return [(ZEditFileSystemItem *)_owner sharedWindowController];
		}
		else
		{
			return nil;
		}
	}
}

- (void)collectUnsavedTextDocuments:(NSMutableArray *)documents
{
	if(_document != nil && [_document isKindOfClass:[ZEditTextDocument class]])
	{
		if([_document isDocumentEdited])
		{
			[documents addObject:_document];
		}
	}
	ZEditFileSystemItem *realItem = (ZEditFileSystemItem *)[self resolveIfSymbolicLinkOrAlias];
	
	if(realItem->_children != nil)
	{
		int i;
		for(i = 0; i < [realItem->_children count]; i++)
		{
			[[realItem->_children objectAtIndex:i] collectUnsavedTextDocuments:documents];
		}
	}
}

- (NSArray *)unsavedTextDocuments
{
	NSMutableArray *result = [NSMutableArray array];
	[self collectUnsavedTextDocuments:result];
	return [[result copy] autorelease];
}

- (void)collectOpenedTextDocuments:(NSMutableArray *)documents
{
	if(_document != nil && [_document isKindOfClass:[ZEditTextDocument class]])
	{
		[documents addObject:_document];
	}
	ZEditFileSystemItem *realItem = (ZEditFileSystemItem *)[self resolveIfSymbolicLinkOrAlias];
	
	if(realItem->_children != nil)
	{
		int i;
		for(i = 0; i < [realItem->_children count]; i++)
		{
			[[realItem->_children objectAtIndex:i] collectOpenedTextDocuments:documents];
		}
	}
}

- (NSArray *)openedTextDocuments
{
	NSMutableArray *result = [NSMutableArray array];
	[self collectOpenedTextDocuments:result];
	return [[result copy] autorelease];
}

- (void)closeAllDocuments
{
	if(_document != nil)
	{
		[self setDocument:nil];
	}
	ZEditFileSystemItem *realItem = (ZEditFileSystemItem *)[self resolveIfSymbolicLinkOrAlias];
	
	if(realItem->_children != nil)
	{
		int i;
		for(i = 0; i < [realItem->_children count]; i++)
		{
			[[realItem->_children objectAtIndex:i] closeAllDocuments];
		}
	}
}

- (void)setDocument:(ZEditDocument *)document
{
	if(_document != document)
	{
		if(_owner == nil && _parent == nil)
		{
			[_document removeObserver:self forKeyPath:@"fileURL"];
		}
		[_document close];
		[_document release];
		_document = [document retain];
		if(_owner == nil && _parent == nil)
		{
			[_document addObserver:self forKeyPath:@"fileURL" options:0 context:nil];
		}
	}
}

- (ZEditDocument *)documentWithError:(NSError **)outError
{
	if(_document == nil)
	{
		ZFileSystemItem *real = [self resolveIfSymbolicLinkOrAlias];
		NSURL *url = [NSURL fileURLWithPath:[real path]];
		
		ZEditDocument *newDocument;
		if([real isDirectory])
		{
			newDocument = [[[ZEditDirectoryDocument alloc] initWithContentsOfURL:url ofType:@"ZEditTextDocument"
														   sharedWindowController:[self sharedWindowController]
														   error:outError] autorelease];;
		}
		else
		{
			newDocument = [[[ZEditTextDocument alloc] initWithContentsOfURL:url
													  ofType:@"ZEditTextDocument"
													  sharedWindowController:[self sharedWindowController]
													  error:outError] autorelease];
		}
		[self setDocument:newDocument];
	}
	return _document;
}

- (ZEditDocument *)document
{
	return [self documentWithError:nil];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:@"fileURL"])
	{
		NSString *path = [[[self document] fileURL] path];
		NSString *parentPath = [path stringByDeletingLastPathComponent];
		[_parentPath release];
		_parentPath = [parentPath retain];
		
		NSString *fileName = [path lastPathComponent];
		if([fileName isEqualToString:@"/"])
		{
			fileName = @"";
		}
		[_fileName release];
		_fileName = [fileName retain];
		[[self sharedWindowController] setRootDirectory:self];
	}
	else
	{
		[super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
	}
}
@end
