//
//  ZEditWindowController.m
//  ZEdit
//
//  Created by 増田 勇樹 on 07/03/15.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "ZEditWindowController.h"
#import "ZEditFileSystemItem.h"
#import "ZEditDocument.h"

struct ContextInfo
{
	id delegate;
	SEL didCloseAllSelector;
	void *contextInfo;
	NSMutableArray *unsavedDocuments; // don't forget to release.
	ZEditDocument *sender; // don't forget to release.
};

@interface ZEditWindowController (Privates)
- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(struct ContextInfo *)contextInfo;
- (void)document:(NSDocument *)doc shouldClose:(BOOL)shouldClose  contextInfo:(struct ContextInfo *)contextInfo;
@end

@implementation ZEditWindowController
- (void)awakeFromNib
{
	[[self window] setFrameAutosaveName:@"DocumentWindow"];
	[scrollView setHasVerticalScroller:YES];
	[scrollView setHasHorizontalRuler:NO];
	if([scrollView documentView] == nil)
	{
		[self setDocument:[self document]];
	}
	if(_rootDirectory != nil)
	{
		[drawer open];
		[fileListOutlineView expandItem:[fileListOutlineView itemAtRow:0]];
		[[[fileListOutlineView enclosingScrollView] contentView] setCopiesOnScroll:YES];
	}
}

- (void)dealloc
{
	[self setRootDirectory:nil];
	[super dealloc];
}

- (void)setDocument:(ZEditDocument *)document
{
	[scrollView setDocumentView:[document view]];
	[super setDocument:document];
}

- (void)setRootDirectory:(ZEditFileSystemItem *)rootDirectory
{
	if(_rootDirectory != rootDirectory)
	{
		[_rootDirectory release];
		_rootDirectory = [rootDirectory retain];
		[_rootDirectory setSharedWindowController:self];
	}
}

- (ZEditFileSystemItem *)rootDirectory
{
	return _rootDirectory;
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
	NSArray *selection = [fileListController selectedObjects];
	if([selection count] > 0)
	{
		NSDocument *document = [[selection objectAtIndex:0] document];
		[document addWindowController:self];
	}
	else
	{
		[[[self rootDirectory] document] addWindowController:self];
	}
}

- (void)closeAllDocuments
{
	if(_rootDirectory != nil)
	{
		[_rootDirectory closeAllDocuments];
	}
	else
	{
		[[self document] close];
	}
}

- (void)canCloseAllDocumentWithDelegate:(id)delegate
		didCloseAllSelector:(SEL)didCloseAllSelector
		contextInfo:(void *)contextInfo
		sender:(ZEditDocument *)sender
{
	NSArray *unsaved;
	if(_rootDirectory != nil)
	{
		unsaved = [_rootDirectory unsavedTextDocuments];
	}
	else
	{
		if([[self document] isDocumentEdited])
		{
			unsaved = [NSArray arrayWithObject:[self document]];
		}
		else
		{
			unsaved = [NSArray array];
		}
	}
	struct ContextInfo *info = NSZoneMalloc(NSDefaultMallocZone(), sizeof(struct ContextInfo));
	info->delegate = delegate;
	info->didCloseAllSelector = didCloseAllSelector;
	info->contextInfo = contextInfo;
	info->unsavedDocuments = [unsaved mutableCopy];
	info->sender = [sender retain];
	
	if([unsaved count] == 0)
	{
		[self sheetDidEnd:nil returnCode:NSAlertAlternateReturn contextInfo:info];
	}
	else
	{
		if([unsaved count] == 1)
		{
			[self sheetDidEnd:nil returnCode:NSAlertDefaultReturn contextInfo:info];
		}
		else
		{
			NSBeginAlertSheet([NSString stringWithFormat:NSLocalizedString(@"ZEditConfirmCloseTitle", nil), [unsaved count]],
							  NSLocalizedString(@"ZEditReviewChangesButton", nil),
							  NSLocalizedString(@"ZEditDiscardChangesButton", nil),
							  NSLocalizedString(@"ZEditCancelButton", nill),
							  [self window],
							  self,
							  @selector(sheetDidEnd:returnCode:contextInfo:),
							  nil,
							  info,
							  NSLocalizedString(@"ZEditConfirmCloseMessage", nil));
		}
	}
}
@end

void FreeContextInfo(struct ContextInfo *info)
{
	[info->unsavedDocuments release];
	[info->sender release];
	NSZoneFree(NSDefaultMallocZone(), info);
}

@implementation ZEditWindowController (Privates)
- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(struct ContextInfo *)contextInfo
{
	[sheet close];
	switch(returnCode)
	{
		case NSAlertDefaultReturn:
			[self document:nil shouldClose:YES contextInfo:contextInfo];
			break;
		case NSAlertAlternateReturn:
			[self closeAllDocuments];
			objc_msgSend(contextInfo->delegate, contextInfo->didCloseAllSelector, contextInfo->sender, YES, contextInfo->contextInfo);
			FreeContextInfo(contextInfo);
			break;
		case NSAlertOtherReturn:
			objc_msgSend(contextInfo->delegate, contextInfo->didCloseAllSelector, contextInfo->sender, NO, contextInfo->contextInfo);
			FreeContextInfo(contextInfo);
			break;
		default:
			break;
	}
}

- (void)document:(NSDocument *)doc shouldClose:(BOOL)shouldClose  contextInfo:(struct ContextInfo *)contextInfo
{
	if(shouldClose)
	{
		if(doc != nil)
		{
			[contextInfo->unsavedDocuments removeObjectAtIndex:0];
		}
		if([contextInfo->unsavedDocuments count] == 0)
		{
			[self closeAllDocuments];
			objc_msgSend(contextInfo->delegate, contextInfo->didCloseAllSelector, contextInfo->sender, YES, contextInfo->contextInfo);
			FreeContextInfo(contextInfo);
		}
		else
		{
			ZEditDocument *doc = [contextInfo->unsavedDocuments objectAtIndex:0];
			[doc addWindowController:self];
			[doc canCloseDocumentWithDelegate:self
				 shouldCloseSelector:@selector(document:shouldClose:contextInfo:)
				 contextInfo:contextInfo];
		}
	}
	else
	{
		objc_msgSend(contextInfo->delegate, contextInfo->didCloseAllSelector, contextInfo->sender, NO, contextInfo->contextInfo);
		FreeContextInfo(contextInfo);
	}
}
@end
