//
//  ZEncodingList.m
//  ZEdit
//
//  Created by 増田 勇樹 on 07/02/23.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "ZEncodingList.h"

#import "ZEncodingMenuController.h"
#import "NSString-ZEditExtension.h"

@interface EncodingListItem : NSObject
{
	NSStringEncoding _encoding;
}

- (id)initWithEncoding:(NSStringEncoding)encoding;

- (NSStringEncoding)encoding;

- (NSString *)name;

- (BOOL)availabilityEditable;

- (void)setAvailability:(BOOL)availability;
- (BOOL)availability;
@end

@implementation ZEncodingList
- (void)dealloc
{
	[_encodingList release];
	[super dealloc];
}

- (NSArray *)encodingList
{
	if(_encodingList == nil)
	{
		NSMutableArray *encodingList = [NSMutableArray array];
		const NSStringEncoding *encodings = [NSString availableStringEncodings];
		NSStringEncoding enc;
		while((enc = *encodings++) != 0)
		{
			EncodingListItem *item = [[[EncodingListItem alloc] initWithEncoding:enc] autorelease];
			[encodingList addObject:item];
		}
		[encodingList sortUsingSelector:@selector(compare:)];
		_encodingList = [encodingList copy];
	}
	return _encodingList;
}

- (IBAction)selectAll:(id)sender
{
	for(EncodingListItem *item in [self encodingList])
	{
		if([item availabilityEditable])
		{
			[item setAvailability:YES];
		}
	}
}

- (IBAction)selectNone:(id)sender
{
	for(EncodingListItem *item in [self encodingList])
	{
		if([item availabilityEditable])
		{
			[item setAvailability:NO];
		}
	}
}

- (IBAction)revertToDefault:(id)sender
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	NSArray *initialEncodings = [[udc initialValues] objectForKey:AVAILABLE_ENCODINGS_KEY];
	for(EncodingListItem *item in [self encodingList])
	{
		if([item availabilityEditable])
		{
			[item setAvailability:[initialEncodings containsObject:[NSNumber numberWithInt:[item encoding]]]];
		}
	}
}
@end

@implementation EncodingListItem
- (id)initWithEncoding:(NSStringEncoding)encoding
{
	self = [super init];
	if(self != nil)
	{
		_encoding = encoding;
	}
	return self;
}

- (NSStringEncoding)encoding
{
	return _encoding;
}

- (NSString *)name
{
	return [NSString localizedNameOfStringEncoding:_encoding];
}

- (BOOL)availabilityEditable
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	int currentEncoding = [[[udc values] valueForKeyPath:DEFAULT_ENCODING_KEY] intValue];
	if(currentEncoding == _encoding)
	{
		return NO;
	}
	else
	{
		return YES;
	}
}

- (void)setAvailability:(BOOL)availability
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	NSArray *udcEncodings = [[[[udc values] valueForKeyPath:AVAILABLE_ENCODINGS_KEY] mutableCopy] autorelease];
	
	NSMutableArray *availableEncodings;
	if([udcEncodings isKindOfClass:[NSMutableArray class]])
	{
		availableEncodings = (NSMutableArray *)udcEncodings;
	}
	else
	{
		availableEncodings = [[[[udc values] valueForKeyPath:AVAILABLE_ENCODINGS_KEY] mutableCopy] autorelease];
	}
	NSNumber *myEncoding = [NSNumber numberWithInt:_encoding];
	if(availability)
	{
		if(![availableEncodings containsObject:myEncoding])
		{
			[availableEncodings addObject:myEncoding];
		}
	}
	else
	{
		[availableEncodings removeObject:myEncoding];
	}
	[[udc values] setValue:availableEncodings forKeyPath:AVAILABLE_ENCODINGS_KEY];
}

- (BOOL)availability
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	NSArray *availableEncodings = [[udc values] valueForKeyPath:AVAILABLE_ENCODINGS_KEY];
	if([availableEncodings containsObject:[NSNumber numberWithInt:_encoding]])
	{
		return YES;
	}
	else
	{
		return NO;
	}
}

- (NSComparisonResult)compare:(EncodingListItem *)item
{
	return [[self name] compare:[item name]];
}
@end
