//
//  ZEncodingMenuController.m
//  ZEdit
//
//  Created by 増田 勇樹 on 07/03/01.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "ZEncodingMenuController.h"

int encodingCompare(NSNumber *item1, NSNumber *item2, void *context)
{
	return [[NSString localizedNameOfStringEncoding:[item1 intValue]] compare:[NSString localizedNameOfStringEncoding:[item2 intValue]]];
}

@implementation ZEncodingMenuController
@synthesize needsUpdate = _needsUpdate;

+ (void)initialize
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	NSMutableDictionary *initialValues = [[[udc initialValues] mutableCopy] autorelease];
	if(initialValues == nil)
	{
		initialValues = [NSMutableDictionary dictionary];
	}

	NSMutableArray *availableEncodings = [NSMutableArray arrayWithObjects:[NSNumber numberWithInt:NSASCIIStringEncoding],
																		  [NSNumber numberWithInt:NSJapaneseEUCStringEncoding],
																		  [NSNumber numberWithInt:NSUTF8StringEncoding],
																		  [NSNumber numberWithInt:NSISOLatin1StringEncoding],
																		  [NSNumber numberWithInt:NSNonLossyASCIIStringEncoding],
																		  [NSNumber numberWithInt:NSShiftJISStringEncoding],
																		  [NSNumber numberWithInt:NSISOLatin2StringEncoding],
																		  [NSNumber numberWithInt:NSUnicodeStringEncoding],
																		  [NSNumber numberWithInt:NSISO2022JPStringEncoding],
																		  [NSNumber numberWithInt:NSMacOSRomanStringEncoding],
																		  [NSNumber numberWithInt:NSWindowsCP1251StringEncoding],
																		  [NSNumber numberWithInt:NSWindowsCP1252StringEncoding],
																		  [NSNumber numberWithInt:NSWindowsCP1253StringEncoding],
																		  [NSNumber numberWithInt:NSWindowsCP1254StringEncoding],
																		  [NSNumber numberWithInt:NSWindowsCP1250StringEncoding], nil];
	NSNumber *defaultEncoding = [NSNumber numberWithInt:[NSString defaultCStringEncoding]];
	if(![availableEncodings containsObject:defaultEncoding])
	{
		[availableEncodings addObject:defaultEncoding];
	}
	[initialValues setValue:availableEncodings forKeyPath:AVAILABLE_ENCODINGS_KEY];
	[initialValues setObject:defaultEncoding forKey:DEFAULT_ENCODING_KEY];
	[udc setInitialValues:initialValues];
}

- (void)awakeFromNib
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	[[udc values] addObserver:self forKeyPath:AVAILABLE_ENCODINGS_KEY options:0 context:nil];
	[[udc values] addObserver:self forKeyPath:DEFAULT_ENCODING_KEY options:NSKeyValueObservingOptionInitial context:nil];
}

- (void)dealloc
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	[[udc values] removeObserver:self];
	[super dealloc];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:AVAILABLE_ENCODINGS_KEY] || [keyPath isEqualToString:DEFAULT_ENCODING_KEY])
	{
		[self setNeedsUpdate:YES];
	}
}

- (void)menuNeedsUpdate:(NSMenu *)menu
{
	if([self needsUpdate])
	{
		NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
		NSArray *availableEncodings = [[udc values] valueForKeyPath:AVAILABLE_ENCODINGS_KEY];
		availableEncodings = [availableEncodings sortedArrayUsingFunction:encodingCompare context:nil];
		
		int i;
		for(i = 0; i < [menu numberOfItems]; i++)
		{
			[menu removeItemAtIndex:0];
		}
		//adding encodings to menu
		for(i = 0; i < [availableEncodings count]; i++)
		{
			NSStringEncoding encoding = [[availableEncodings objectAtIndex:i] intValue];
			NSString *name = [NSString localizedNameOfStringEncoding:encoding];
			
			NSMenuItem *item = [[[NSMenuItem alloc] initWithTitle:name action:@selector(changeEncoding:) keyEquivalent:@""] autorelease];
			[item setTag:encoding];
			[menu addItem:item];
		}
		[self setNeedsUpdate:NO];
	}
}
@end
