#import <Cocoa/Cocoa.h>

@class DRFileNotificationSubscription;

#define FileSystemItemRemovedFromTreeNotification @"FileSystemItemRemovedFromTreeNotification"

typedef enum _FileSystemItemTypeFlags
{
	FileSystemItemIsDirectory = 1,
	FileSystemItemIsSymbolicLink = 2,
	FileSystemItemIsAlias = 4,
	FileSystemItemIsPackage = 8,
	FileSystemItemIsInvisible = 16
} FileSystemItemTypeFlags;

@interface ZFileSystemItem : NSObject
{
	FileSystemItemTypeFlags _flags;
    NSString *_parentPath;
	NSString *_fileName;
	
	ZFileSystemItem *_owner;
    ZFileSystemItem *_parent;
    NSMutableArray *_children;
	
	BOOL _includesInvisibleChildren;
	BOOL _treatFilePackagesAsDirectories;
	
	ZFileSystemItem *_resolvedItem;
	
	DRFileNotificationSubscription *_directoryNotification;
	
	NSImage *_icon;
}

+ (id)rootItemWithPath:(NSString *)path;

- (id)initAsRootWithPath:(NSString *)path;
- (id)initAsRootWithPath:(NSString *)path owner:(ZFileSystemItem *)owner;
- (id)initWithPath:(NSString *)path parent:(ZFileSystemItem *)parent;
- (id)initWithPath:(NSString *)path parent:(ZFileSystemItem *)parent owner:(ZFileSystemItem *)owner;

- (id)copyWithZone:(NSZone *)zone;

- (BOOL)isEqual:(ZFileSystemItem *)item;

- (NSString *)path;

// permissions
- (BOOL)readable;
- (BOOL)writable;
- (BOOL)executable;

// icon
- (NSImage *)icon;

// attributes
- (NSDictionary *)fileAttributes;
- (BOOL)extensionHidden;

// file type
- (FileSystemItemTypeFlags)typeFlags;
- (BOOL)isDirectory;
- (BOOL)isSymbolicLink;
- (BOOL)isAlias;
- (BOOL)isPackage;
- (BOOL)isInvisible;

// link
- (ZFileSystemItem *)resolveIfSymbolicLinkOrAlias;
- (ZFileSystemItem *)resolveSymbolicLink;
- (ZFileSystemItem *)resolveAlias;

// display
- (void)setIncludesInvisibleChildren:(BOOL)include;
- (BOOL)includesInvisibleChildren;

- (void)setTreatsFilePackagesAsDirectories:(BOOL)asDirectory;
- (BOOL)treatsFilePackagesAsDirectories;

// file name
- (BOOL)canRename;
- (BOOL)canRenameTo:(NSString *)fileName;
- (void)setFileName:(NSString *)fileName;
- (NSString *)fileName;

- (BOOL)canChangeDisplayNameTo:(NSString *)newDisplayName;
- (void)setDisplayName:(NSString *)newDisplayName;
- (NSString *)displayName;

// move
- (BOOL)moveToDirectory:(NSString *)directoryPath;
- (BOOL)moveToTrash;

// create
- (NSString *)nameForUntitledChildFile;
- (NSString *)nameForUntitledChildDirectory;

- (ZFileSystemItem *)createChildDirectoryNamed:(NSString *)directoryName;
- (ZFileSystemItem *)createChildFileNamed:(NSString *)fileName;

- (ZFileSystemItem *)parent;

// children
- (NSArray *)children;
- (int)countOfChildren;
- (ZFileSystemItem *)objectInChildrenAtIndex:(unsigned int)index;

- (NSIndexPath *)indexPathFromRoot;

- (BOOL)isLeaf;
@end
