//
//  ZFileSystemTreeController.m
//  ZEdit
//
//  Created by 増田 勇樹 on 07/03/04.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "ZFileSystemTreeController.h"

#import "ZFileSystemItem.h"

@implementation NSOutlineView (EnterEndsEditing)
- (void)textDidEndEditing:(NSNotification*)notification
{
    if ([[[notification userInfo] objectForKey:@"NSTextMovement"] intValue] == NSReturnTextMovement)
	{
        NSMutableDictionary *newUserInfo = [NSMutableDictionary dictionaryWithDictionary:[notification userInfo]];
        [newUserInfo setObject:[NSNumber numberWithInt:NSIllegalTextMovement] forKey:@"NSTextMovement"];
        NSNotification *newNotification = [NSNotification notificationWithName:[notification name] object:[notification object] userInfo:newUserInfo];
        [super textDidEndEditing:newNotification];
        [[self window] makeFirstResponder:self];
    } 
	else
	{
        [super textDidEndEditing:notification];
    }
}
@end

@implementation ZFileSystemTreeController
- (ZFileSystemItem *)firstObjectInSelectedObjects
{
	NSArray *selectedObjects = [self selectedObjects];
	if([selectedObjects count] == 0)
	{
		return nil;
	}
	else
	{
		return [selectedObjects objectAtIndex:0];
	}
}

- (ZFileSystemItem *)getNewFileLocation
{
	ZFileSystemItem *selection = [self firstObjectInSelectedObjects];
	
	if([selection isDirectory] || [[selection resolveIfSymbolicLinkOrAlias] isDirectory])
	{
		return selection;
	}
	else
	{
		return [selection parent];
	}
}

- (NSArray *)removeTargets
{
	NSMutableArray *result = [NSMutableArray array];
	NSEnumerator *selectedObjects = [[self selectedObjects] objectEnumerator];
	
	ZFileSystemItem *item;
	while((item = [selectedObjects nextObject]) != nil)
	{
		if([item parent] == nil)
		{
			[result removeAllObjects];
			return result;
		}
		else
		{
			if([[item parent] writable] && [item writable])
			{
				[result addObject:item];
			}
		}
	}
	return [[result copy] autorelease];
}

- (BOOL)canCreateFileOrDirectory
{
	ZFileSystemItem *selection = [self getNewFileLocation];
	selection = [selection resolveIfSymbolicLinkOrAlias];
	return [selection isDirectory] && [selection writable];
}

- (IBAction)createUntitledFile:(id)sender
{
	ZFileSystemItem *parent = [self getNewFileLocation];
	if(parent != nil)
	{
		[parent createChildFileNamed:[parent nameForUntitledChildFile]];
	}
}

- (IBAction)createUntitledDirectory:(id)sender
{
	ZFileSystemItem *parent = [self getNewFileLocation];
	if(parent != nil)
	{
		[parent createChildDirectoryNamed:[parent nameForUntitledChildDirectory]];
	}
}

- (BOOL)canRemoveSelection
{
	return [[self removeTargets] count] > 0;
}

-(void)removeSelection:(id)sender
{
	NSArray *removeTargets = [self removeTargets];
	if([removeTargets count] > 0)
	{
		[removeTargets retain];
		NSBeginAlertSheet(NSLocalizedString(@"ZEditMoveToTrashTitle", nil),
						  nil,
						  nil,
						  NSLocalizedString(@"ZEditCancelButton", nill),
						  mainWindow,
						  self,
						  @selector(sheetDidEnd:returnCode:contextInfo:),
						  nil,
						  removeTargets,
						  [NSString stringWithFormat:NSLocalizedString(@"ZEditMoveToTrashMessage", nil), [removeTargets count] > 1? @"s": @""]);
	}
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	NSArray *removeTargets = [(NSArray *)contextInfo autorelease];
	if(returnCode == NSAlertDefaultReturn)
	{
		int i;
		for(i = 0; i < [removeTargets count]; i++)
		{
			[[removeTargets objectAtIndex:i] moveToTrash];
		}
		[self setSelectionIndexPath:nil];
	}
}

- (void)selectFile:(ZFileSystemItem *)item;
{
	[self setSelectionIndexPath:[item indexPathFromRoot]];
}
@end
