//
//  ZMode.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/11/02.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZMode.h"

#import <ZScannerKit/ZScannerKit.h>

#import "ZModeLoader.h"
#import "ZSyntax.h"

NSString *ZModeResourceDirectory = @"Modes";
NSString *ZModeFileExtension = @"zmode";
NSString *ZModeDescriptionFileName = @"mode.plist";
NSString *ZModeDefaultModeName = @"Plain Text";

/****************************************
* Keys for dictionary in mode.plist File
*****************************************/
NSString *ZModeModeNameKey = @"name"; //string.
NSString *ZModeExtendsKey = @"extends"; //string.
NSString *ZModeFileNameMatchKey = @"fileNameMatch"; //regular expression
NSString *ZModeAutoIndentKey = @"autoIndent"; //BOOL
NSString *ZModeTabWidthKey = @"tabWidth"; //integer

@interface ZMode (Internal)
- (unsigned int)indentWidthOfLine:(NSString *)line;
@end

@implementation ZMode
@synthesize name = _name,
			fileNameMatch = _fileNameMatch,
			autoIndent = _autoIndent,
			tabWidth = _tabWidth;

+ (NSArray *)modes
{
	return [[ZModeLoader sharedModeLoader] modes];
}

+ (ZMode *)modeWithName:(NSString *)name
{
	for(ZMode *m in [self modes])
	{
		if([[m name] isEqualToString:name])
		{
			return m;
		}
	}
	return nil;
}

+ (ZMode *)modeForFile:(NSString *)fileName
{
	fileName = [fileName lastPathComponent];
	for(ZMode *m in [self modes])
	{
		ZScanner *s = [ZScanner scanner];
		[s addRuleWithIdentifier:self regularExpression:[m fileNameMatch]];
		
		ZScannerMatch *match = [s matchInString:fileName range:NSMakeRange(0, [fileName length])];
		
		if((match != nil) && ([match range].length == [fileName length]))
		{
			return m;
		}
	}
	return nil;
}

- (id)initWithDescription:(NSDictionary *)description
{
	self = [super init];
	if(self != nil)
	{
		_name = [[description objectForKey:ZModeModeNameKey] retain];
		_fileNameMatch = [[description objectForKey:ZModeFileNameMatchKey] retain];
		_autoIndent = [[description objectForKey:ZModeAutoIndentKey] boolValue];
		_tabWidth = [[description objectForKey:ZModeTabWidthKey] intValue];
	}
	return self;
}

- (void)dealloc
{
	[_name release];
	[_fileNameMatch release];
	[super dealloc];
}

- (ZSyntax *)syntax
{
	NSDictionary *desc = [[ZModeLoader sharedModeLoader] syntaxDescriptionForMode:self];
	return [[[ZSyntax alloc] initWithDescription:desc] autorelease];
}

- (NSParagraphStyle *)paragraphStyleForFont:(NSFont *)font
{
	NSDictionary *att = [NSDictionary dictionaryWithObject:font forKey:NSFontAttributeName];
	float tabInterval = [@" " sizeWithAttributes:att].width * [self tabWidth];
	NSMutableParagraphStyle *pstyle = [[[NSParagraphStyle defaultParagraphStyle] mutableCopy] autorelease];
	for(NSTextTab *t in [pstyle tabStops])
	{
		[pstyle removeTabStop:t];
	}
	[pstyle setDefaultTabInterval:tabInterval];
	return [[pstyle copy] autorelease];
}

- (void)applyIndentInTextStorage:(NSTextStorage *)storage location:(unsigned int)location
{
	if(![self autoIndent])
	{
		return;
	}
	NSMutableString *string = [storage mutableString];
	unsigned int indentWidth = 0;
	
	NSString *prevLine = [string substringWithRange:[string lineRangeForRange:NSMakeRange(location-1, 0)]];
	indentWidth = [self indentWidthOfLine:prevLine];

	unsigned int at = location;
	unsigned int t, w;
	unsigned int tabWidth = [self tabWidth];
	for(t = indentWidth / tabWidth;t > 0; t--)
	{
		[string insertString:@"\t" atIndex:at];
		at++;
	}
	for(w = indentWidth % tabWidth;w > 0; w--)
	{
		[string insertString:@" " atIndex:at];
		at++;
	}
}
@end

@implementation ZMode (Internal)
- (unsigned int)indentWidthOfLine:(NSString *)line
{
	unichar tab = [@"\t" characterAtIndex:0];
	unichar space = [@" " characterAtIndex:0];
	unichar buf[[line length]];
	
	[line getCharacters:buf];
	int w = 0;
	int i;
	for(i = 0; i < [line length]; i++)
	{
		if(buf[i] == tab)
		{
			w += [self tabWidth];
		}
		else if(buf[i] == space)
		{
			w++;
		}
		else
		{
			break;
		}
	}
	return w;
}
@end