//
//  ZModeLoader.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/11/04.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZModeLoader.h"

#import "ZSyntax.h"
#import "ZMode.h"

static void get_descs(NSDictionary **modes, NSDictionary **syntaxes);
static NSDictionary *load_plist(NSString *file, NSBundle *bundle);
static NSDictionary *extend_descs(NSString *extendsKey, NSDictionary *descs);
static NSDictionary *extend_desc(NSString *extendsKey, NSDictionary *d, NSDictionary *others);
static NSDictionary *extend_dictionary(NSDictionary *c, NSDictionary *p);
static NSArray *extend_array(NSArray *c, NSArray *p);

static ZModeLoader *sharedModeLoader = nil;

@interface ZModeLoader (Internal)
- (void)loadModes;
@end

@implementation ZModeLoader
@synthesize modes = _modes;

+ (ZModeLoader *)sharedModeLoader
{
	if(sharedModeLoader == nil)
	{
		sharedModeLoader = [[ZModeLoader alloc] init];
	}
	return sharedModeLoader;
}

- (id)init
{
	self = [super init];
	if(self != nil)
	{
		[self loadModes];
	}
	return self;
}

- (void)dealloc
{
	[_modes release];
	[super dealloc];
}

- (NSDictionary *)syntaxDescriptionForMode:(ZMode *)mode
{
	return [_syntaxDescriptions objectForKey:[mode name]];
}
@end


@implementation ZModeLoader (Internal)
- (void)loadModes
{
	if(_modes == nil)
	{
		NSDictionary *modeDescs = nil;
		NSDictionary *syntaxDescs = nil;
		get_descs(&modeDescs, &syntaxDescs);
				
		_syntaxDescriptions = [syntaxDescs retain];
		
		NSMutableArray *modes = [NSMutableArray array];
		for(NSString *name in modeDescs)
		{
			NSDictionary *d = [modeDescs objectForKey:name];
			ZMode *m = [[[ZMode alloc] initWithDescription:d] autorelease];
			if([[m name] isEqualToString:ZModeDefaultModeName])
			{
				[modes addObject:m];
			}
			else
			{
				[modes insertObject:m atIndex:0];
			}
		}
		_modes = [modes copy];
	}
}
@end

static void get_descs(NSDictionary **modes, NSDictionary **syntaxes)
{
	*modes = [NSMutableDictionary dictionary];
	*syntaxes = [NSMutableDictionary dictionary];
	
	NSBundle *mainBundle = [NSBundle mainBundle];
	NSArray *paths = [mainBundle pathsForResourcesOfType:ZModeFileExtension inDirectory:ZModeResourceDirectory];
	for(NSString *p in paths)
	{
		NSBundle *b = [NSBundle bundleWithPath:p];
		if(b == nil)
		{
			[NSException raise:@"ZModeException" format:@"can't load mode bundle"];
		}
		NSDictionary *mdesc = load_plist(ZModeDescriptionFileName, b);
		
		NSString *mname = [mdesc objectForKey:ZModeModeNameKey];
		if(mname == nil)
		{
			[NSException raise:@"ZModeException" format:@"mode name is missing in %@ at %@.", ZModeDescriptionFileName, [b bundlePath]];
		}
		
		[(NSMutableDictionary *)*modes setObject:mdesc forKey:mname];
		
		NSDictionary *sdesc = load_plist(ZSyntaxDescriptionFileName, b);
		[(NSMutableDictionary *)*syntaxes setObject:sdesc forKey:mname];
	}
	
	*modes = extend_descs(ZModeExtendsKey, *modes);
	*syntaxes = extend_descs(ZSyntaxExtendsKey, *syntaxes);
}

static NSDictionary *extend_descs(NSString *extendsKey, NSDictionary *descs)
{
	NSMutableDictionary *extended = [NSMutableDictionary dictionary];
	for(NSString *name in [descs allKeys])
	{
		NSDictionary *new = extend_desc(extendsKey, [descs objectForKey:name], descs);
		[extended setObject:new forKey:name];
	}
	return extended;
}

static NSDictionary *load_plist(NSString *file, NSBundle *bundle)
{
	NSString *base = [file stringByDeletingPathExtension];
	NSString *ext = [file pathExtension];
	NSString *path = [bundle pathForResource:base ofType:ext inDirectory:nil];
	if(path == nil)
	{
		[NSException raise:@"ZModeException" format:@"file %@ not found in mode bundle %@.", file, [bundle bundlePath]];
	}
	NSDictionary *plist = [NSDictionary dictionaryWithContentsOfFile:path];
	if(plist == nil)
	{
		[NSException raise:@"ZModeException" format:@"%@ int %@ is not in proper format.", file, [bundle bundlePath]];
	}	
	return plist;
}

NSDictionary *extend_desc(NSString *extendsKey, NSDictionary *d, NSDictionary *others)
{
	NSString *superName = [d objectForKey:extendsKey];
	if(superName != nil)
	{
		return extend_dictionary(d, extend_desc(extendsKey, [others objectForKey:superName], others));
	}
	else
	{
		return d;
	}
}

static NSDictionary *extend_dictionary(NSDictionary *c, NSDictionary *p)
{
	if(c == nil)
	{
		return p;
	}
	else if(p == nil)
	{
		return c;
	}
	else
	{
		NSMutableDictionary *extended = [[p mutableCopy] autorelease];
		for(NSString *k in [c allKeys])
		{
			id cv = [c objectForKey:k];	
			if([cv isKindOfClass:[NSDictionary class]])
			{
				id pv = [p objectForKey:k];
				[extended setObject:extend_dictionary(cv, pv) forKey:k];
			}
			else if([cv isKindOfClass:[NSArray class]])
			{
				id pv = [p objectForKey:k];
				[extended setObject:extend_array(cv, pv) forKey:k];
			}
			else
			{
				[extended setObject:cv forKey:k];
			}
		}
		return extended;
	}
}

static NSArray *extend_array(NSArray *c, NSArray *p)
{
	if(c == nil)
	{
		return p;
	}
	else if(p == nil)
	{
		return c;
	}
	else
	{
		return [p arrayByAddingObjectsFromArray:c];
	}
}