//
//  ZModeTextView.m
//  sEdit
//
//  Created by 増田 勇樹 on 04/12/05.
//  Copyright 2004 __MyCompanyName__. All rights reserved.
//

#import "ZModeTextView.h"

#import "ZMode.h"
#import "ZSyntax.h"
#import "ZBindingManager.h"

@implementation ZModeTextView
@synthesize statusBarMessage = _statusBarMessage, syntax = _syntax;

+ (NSDictionary *)defaultColors
{
	NSMutableDictionary *colors = [NSMutableDictionary dictionary];
	[colors setObject:[NSColor blackColor] forKey:ColorsTextKey];
	[colors setObject:[NSColor whiteColor] forKey:ColorsBackgroundKey];
	[colors setObject:[NSColor orangeColor] forKey:ColorsCommentKey];
	[colors setObject:[NSColor orangeColor] forKey:ColorsPreprocessorKey];
	[colors setObject:[NSColor redColor] forKey:ColorsStringKey];
	[colors setObject:[NSColor grayColor] forKey:ColorsParenthesisKey];
	[colors setObject:[NSColor cyanColor] forKey:ColorsKeywordKey];
	[colors setObject:[NSColor greenColor] forKey:ColorsCharacterKey];
	[colors setObject:[NSColor blueColor] forKey:ColorsNumberKey];
	return [[colors copy] autorelease];
}

- (id)initWithFrame:(NSRect)rect
{
	self = [super initWithFrame:rect];
	if(self != nil)
	{
		[[self textStorage] setDelegate:self];
		[self setSmartInsertDeleteEnabled:NO];
		
		_colorsBindingManager = [[ZBindingManager managerForObject:self keyPath:@"colors"] retain];
		_antialiasBindingManager = [[ZBindingManager managerForObject:self keyPath:@"shouldAntialias"] retain];
		
		NSNotificationCenter *dnc = [NSNotificationCenter defaultCenter];
		[dnc addObserver:self selector:@selector(selectionDidChanged:) name:NSTextViewDidChangeSelectionNotification object:self];
	}
	return self;
}

- (void)dealloc
{
	NSNotificationCenter *dnc = [NSNotificationCenter defaultCenter];
	[dnc removeObserver:self];
	[_colorsBindingManager release];
	[_antialiasBindingManager release];
	
	[[self textStorage] setDelegate:nil];
	[_colors release];
	[_mode release];
	[super dealloc];
}

- (BOOL)validateUserInterfaceItem:(NSObject <NSValidatedUserInterfaceItem>*)anItem
{
	if([anItem action] == @selector(changeMode:))
	{
		NSString *menuModeName = [[[ZMode modes] valueForKey:@"name"] objectAtIndex:[anItem tag]];
		ZMode *current = [self mode];
		
		if([menuModeName isEqualToString:[current name]])
		{
			if([anItem isKindOfClass:[NSMenuItem class]])
			{
				[(NSMenuItem *)anItem setState:NSOnState];
			}
			return NO;
		}
		else
		{
			if([(NSObject *)anItem isKindOfClass:[NSMenuItem class]])
			{
				[(NSMenuItem *)anItem setState:NSOffState];
			}
			return YES;
		}
	}
	else
	{
		return [super validateUserInterfaceItem:anItem];
	}
}

- (void)bind:(NSString *)binding toObject:(id)object withKeyPath:(NSString *)keyPath options:(NSDictionary *)options
{	
	if([binding isEqualToString:@"colors"])
	{
		[_colorsBindingManager bindToObject:object withKeyPath:keyPath options:options];
	}
	else if([binding isEqualToString:@"shouldAntialias"])
	{
		[_antialiasBindingManager bindToObject:object withKeyPath:keyPath options:options];
	}
	else
	{
		[super bind:binding toObject:object withKeyPath:keyPath options:options];
	}
}

- (void)unbind:(NSString *)keyPath
{
	if([keyPath isEqualToString:@"colors"])
	{
		[_colorsBindingManager unbind];
	}
	else if([keyPath isEqualToString:@"shouldAntialias"])
	{
		[_antialiasBindingManager unbind];
	}
	else
	{
		[super unbind:keyPath];
	}
}

- (void)updateParagraphStyle
{
	if([self mode] != nil && [self font] != nil)
	{
		NSParagraphStyle *pstyle = [[self mode] paragraphStyleForFont:[self font]];
		[self setDefaultParagraphStyle:pstyle];
		
		NSMutableDictionary *att = [[[self typingAttributes] mutableCopy] autorelease];
		[att setObject:pstyle forKey:NSParagraphStyleAttributeName];
		[self setTypingAttributes:att];
		
		NSTextStorage *storage = [self textStorage];
		
		[storage addAttribute:NSParagraphStyleAttributeName value:pstyle range:NSMakeRange(0, [storage length])];
	}
}

- (void)setMode:(ZMode *)mode
{
	if(_mode != mode)
	{
		[_mode release];
		_mode = [mode retain];
		[_syntax release];
		_syntax = [[mode syntax] retain];
		
		[self updateParagraphStyle];
		[_syntax applyToTextStorage:[self textStorage]
					   editedRange:NSMakeRange(0, [[self textStorage] length])
					   changeInLength:[[self textStorage] length]
					   colors:[self colors]];
	}
}

- (ZMode *)mode
{
	return [[_mode retain] autorelease];
}

- (IBAction)changeMode:(id)sender
{
	ZMode *mode = [[ZMode modes] objectAtIndex:[sender tag]];
	[self setMode:mode];
}

/* Set colors for syntax highlightening.
   Keys used are declared in ZSyntax.h.
*/
- (void)setColors:(NSDictionary *)colors
{
	if(_colors != colors)
	{
		[_colors release];
		_colors = [colors retain];
		[[self syntax] applyColors:_colors
					   textStorage:[self textStorage]
					   range:NSMakeRange(0, [[self textStorage] length])];
	}
}

- (NSDictionary *)colors
{
	if(_colors != nil)
	{
		return [[_colors retain] autorelease];
	}
	else
	{
		return [ZModeTextView defaultColors];
	}
}

- (void)setFont:(NSFont *)font
{
	[super setFont:font];
	[self updateParagraphStyle];
}

- (void)setShouldAntialias:(BOOL)antialias
{
	if(_antialias != antialias)
	{
		_antialias = antialias;
		[self setNeedsDisplay:YES];
	}
}

- (BOOL)shouldAntialias
{
	return _antialias;
}

- (void)insertNewline:(id)sender
{
	[super insertNewline:sender];
	[[self mode]applyIndentInTextStorage:[self textStorage] location:[self selectedRange].location];
}

// delegate methods of text view
- (void)selectionDidChanged:(NSNotification *)aNotification
{
	NSString *string = [self string];
	unsigned int currentLine;
	unsigned int lines = [string getLineNumber:&currentLine atIndex:[self selectedRange].location];

	[self setStatusBarMessage:[NSString stringWithFormat:@"%d:%d", currentLine, lines]];
}

- (void)textStorageDidProcessEditing:(NSNotification *)aNotification
{
	NSTextStorage *storage = [self textStorage];

	if([storage editedMask] & NSTextStorageEditedCharacters)
	{
		NSRange edited = [storage editedRange];
		NSInteger cil = [storage changeInLength];
		[[self syntax] applyToTextStorage:storage
					   editedRange:edited
					   changeInLength:[storage changeInLength]
					   colors:[self colors]];
		[[self syntax] showMatchIn:storage
					   editedRange:edited
					   changeInLength:cil
					   selectedTextAttributes:[self selectedTextAttributes]];
	}
}

- (void)drawRect:(NSRect)rect
{
	BOOL oldAntialias = [[NSGraphicsContext currentContext] shouldAntialias];
	BOOL antialias = [self shouldAntialias];
	if(antialias != oldAntialias)
	{
		[[NSGraphicsContext currentContext] setShouldAntialias:antialias];
	}
	[super drawRect:rect];
	if(antialias != oldAntialias)
	{
		[[NSGraphicsContext currentContext] setShouldAntialias:oldAntialias];
	}
}

- (NSUndoManager *)undoManager
{
	return [[self window] undoManager];
}
@end
