//
//  ZParseTree.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/10/09.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZParseTree.h"

#import "ZParser.h"
#import "ZParseTreeAction.h"

@implementation ZParseTree
@synthesize parser = _parser,
			triggerLength = _triggerLength,
			subtrees = _subtrees;
			
+ (ZParseTree *)treeWithParser:(ZParser *)parser
{
	return [[[ZParseTree alloc] initWithParser:parser] autorelease];
}

- (id)initWithParser:(ZParser *)parser
{
	self = [super init];
	if(self != nil)
	{
		_parser = [parser retain];
	}
	return self;
}

- (void)dealloc
{
	[_parser release];
	[_subtrees release];
	[super dealloc];
}

- (void)setRange:(NSRange)range
{
	if(_range.location != range.location)
	{
		_range.location = range.location;
	}
	if(_range.length != range.length)
	{
		_range.length = range.length;
	}
}

- (NSRange)range
{
	return _range;
}

- (NSRange)triggerRange
{
	return NSMakeRange([self range].location, [self triggerLength]);
}

- (NSMutableArray *)subtrees
{
	return [self subtreesCreateIfNeeded:YES];
}

- (NSMutableArray *)subtreesCreateIfNeeded:(BOOL)create
{
	if(_subtrees == nil && create)
	{
		_subtrees = [[NSMutableArray array] retain];
	}
	return _subtrees;
}

- (void)shiftBy:(NSInteger)shift
{
	NSRange range = [self range];
	range.location += shift;
	[self setRange:range];
	
	for(ZParseTree *sub in [self subtrees])
	{
		[sub shiftBy:shift];
	}
}

- (void)runActionInRange:(NSRange)range context:(void *)context
{
	range = NSIntersectionRange(range, [self range]);
	if(range.length > 0)
	{
		for(id <ZParseTreeAction> a in [[self parser] actions])
		{
			[a runWithTree:self range:range context:context];
		}
		for(ZParseTree *sub in [self subtrees])
		{
			[sub runActionInRange:range context:context];
		}
	}
}

- (NSString *)description
{
	return [NSString stringWithFormat:@"<%@ : range=%@ parser=%@ subtrees=%@>",
					 NSStringFromClass([self class]),
					 NSStringFromRange([self range]),
					 [[self parser] description],
					 [[self subtrees] description]];
}
@end
