//
//  ZParser.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/10/09.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZParser.h"

#import "ZParseTree.h"
#import "ZStringEdit.h"

@implementation ZParser
@synthesize trigger = _trigger, actions = _actions;

- (id)init
{
	self = [super init];
	if(self != nil)
	{
		_actions = [[NSMutableArray array] retain];
	}
	return self;
}

- (void)dealloc
{
	[_actions release];
	[super dealloc];
}

- (ZParseTree *)parse:(unichar *)characters length:(unsigned int)length triggerRange:(NSRange)triggerRange
{
	ZParseTree *tree = [ZParseTree treeWithParser:self];
	[tree setRange:triggerRange];
	[tree setTriggerLength:triggerRange.length];
	
	NSRange editedRange = NSMakeRange(NSMaxRange(triggerRange), 0);
	
	ZStringEdit *edit = [ZStringEdit editInRange:editedRange changeInLength:0];
	[self reparse:tree edit:edit newCharacters:characters length:length changedRange:NULL];
	return tree;
}

- (ReparseResult)reparse:(ZParseTree *)tree
				 edit:(ZStringEdit *)edit
				 newCharacters:(unichar *)newCharacters
				 length:(unsigned int)length
				 changedRange:(NSRange *)changedRange
{
	if([edit editedRange].location > NSMaxRange([tree range]))
	{
		return ReparseSucceed;
	}
	else if([tree range].location > NSMaxRange([edit editedRange]))
	{
		[tree shiftBy:[edit changeInLength]];
		return ReparseShifted;
	}
	else
	{
		ZExtendRange(changedRange, [edit translatedRange:[tree range]]);
		return ReparseFailed;
	}
}
@end

void ZExtendRange(NSRange *r1, NSRange r2)
{
	if(r1 != NULL)
	{
		*r1 = NSUnionRange(*r1, r2);
	}
}
