#import "ZPreferenceController.h"
#import "ZFontUnarchiver.h"
#import "ZModeTextView.h"
#import "ZSyntax.h"
#import "NSString-ZEditExtension.h"
#import "ZEncodingNameUnarchiver.h"
#import "ZArrayTransformer.h"

@implementation ZPreferenceController
+ (void)initialize
{
	//set custom transformers
	NSValueTransformer *t1 = [[[ZFontUnarchiver alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:t1 forName:@"ZFontUnarchiver"];
	NSValueTransformer *t2 = [[[FontNameUnarchiver alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:t2 forName:@"ZFontNameUnarchiver"];
	ZEncodingNameUnarchiver *t3 = [[[ZEncodingNameUnarchiver alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:t3 forName:@"ZEncodingNameUnarchiver"];
	ZArrayTransformer *t4 = [ZArrayTransformer transFormerWithTransformer:t3];
	[NSValueTransformer setValueTransformer:t4 forName:@"ZEncodingNameArrayUnarchiver"];
	
	//set UserDefaultController behavier
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	[udc setAppliesImmediately:NO];
	
	//set initialValues values	
	NSMutableDictionary *initialValues = [[[udc initialValues] mutableCopy] autorelease];
	if(initialValues == nil)
	{
		initialValues = [NSMutableDictionary dictionary];
	}
	NSFont *font = [NSFont systemFontOfSize:[NSFont smallSystemFontSize]];
	[initialValues setObject:[t1 reverseTransformedValue:font] forKey:PREFERENCE_FONT_KEY];
	
	NSNumber *antialiasText = [NSNumber numberWithInt:YES];
	[initialValues setObject:antialiasText forKey:PREFERENCE_ANTI_ALIAS_KEY];
	
	NSDictionary *colors = [ZModeTextView defaultColors];
	NSMutableDictionary *archivedColors = [NSMutableDictionary dictionary];
	
	NSValueTransformer *u = [NSValueTransformer valueTransformerForName:@"NSUnarchiveFromData"];
	for(NSString *key in [colors allKeys])
	{
		[archivedColors setObject:[u reverseTransformedValue:[colors objectForKey:key]] forKey:key];
	}
	[initialValues setObject:archivedColors forKey:PREFERENCE_COLORS_KEY];
	
	[udc setInitialValues:initialValues];
}

- (IBAction)applyPreference:(id)sender
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	[udc save:self];
	[preferenceWindow orderOut:self];
	NSFontManager *fm = [NSFontManager sharedFontManager];
	[[fm fontPanel:NO] orderOut:self];
}

- (IBAction)cancelPreference:(id)sender
{
	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	[udc revert:self];
	[preferenceWindow orderOut:self];
	NSFontManager *fm = [NSFontManager sharedFontManager];
	[[fm fontPanel:NO] orderOut:self];
}

- (IBAction)changeFont:(id)sender
{	NSUserDefaultsController *udc = [NSUserDefaultsController sharedUserDefaultsController];
	id values = [udc values];
	NSValueTransformer *vt = [NSValueTransformer valueTransformerForName:@"ZFontUnarchiver"];
	NSFont *old = [vt transformedValue:[values valueForKey:PREFERENCE_FONT_KEY]];
	NSFont *new = [sender convertFont:old];
	[values setValue:[vt reverseTransformedValue:new] forKey:PREFERENCE_FONT_KEY];
}
@end
