//
//  ZStateParser.m
//  ZEdit
//
//  Created by 増田 勇樹 on 08/10/09.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "ZStateParser.h"

#import "ZKeywordParser.h"
#import "ZParseTree.h"
#import "ZParseTreeAction.h"
#import "ZStringEdit.h"

static NSRange dirtyIndexRange(NSArray *subtrees, ZStringEdit *edit, unichar *newCharacters, unsigned int length, NSRange *changedRange);
static BOOL reuse(NSMutableArray *subtrees, NSUInteger cursor, NSUInteger indexCursor, NSRange *changedRange);
static void setTreeEnd(ZParseTree *tree, NSUInteger end, ZStringEdit *edit, NSRange *changedRange);

@implementation ZStateParser
- (id)initWithBegin:(NSString *)begin
	  end:(NSString *)end
	  subparsers:(NSArray *)subperesrs
{
	self = [super init];
	if(self != nil)
	{
		[self setTrigger:begin];
		
		_scanner = [[ZScanner scanner] retain];
		
		for(ZParser *sub in subperesrs)
		{
			[_scanner addRuleWithIdentifier:sub regularExpression:[sub trigger]];
		}
		
		NSString *endRegex;
		if(end != nil)
		{
			endRegex = [NSString stringWithFormat:@"(%@)|\\z", end];
		}
		else
		{
			endRegex = [NSString stringWithFormat:@"\\z"];
		}
		_endParser = [[ZKeywordParser parserWithRegex:endRegex] retain];
		[_scanner addRuleWithIdentifier:_endParser regularExpression:[_endParser trigger]];
	}
	return self;
}

- (void)dealloc
{
	[_scanner release];
	[_endParser release];
	[super dealloc];
}

- (ZParseTree *)nextElementIn:(unichar *)string length:(unsigned int)length cursor:(NSUInteger *)cursor
{
	ZScannerMatch *match = [_scanner matchInCharacters:string length:length range:NSMakeRange(*cursor, length - *cursor)];
	ZParser *parser = [match identifier];
	ZParseTree *next = [parser parse:string length:length triggerRange:[match range]];
	if(next != nil)
	{
		*cursor = NSMaxRange([next range]);
	}
	else
	{
		*cursor = NSMaxRange([match range]);
	}
	return next;
}

- (ReparseResult)reparse:(ZParseTree *)tree
				 edit:(ZStringEdit *)edit
				 newCharacters:(unichar *)newCharacters
				 length:(unsigned int)length
				 changedRange:(NSRange *)changedRange
{
	ReparseResult r = [super reparse:tree edit:edit newCharacters:newCharacters length:length changedRange:NULL];
	if(r == ReparseSucceed || r == ReparseShifted)
	{
		return r;
	}
	else if([edit editedRange].location < NSMaxRange([tree triggerRange]))
	{
		ZExtendRange(changedRange, [edit translatedRange:[tree range]]);
		return ReparseFailed;
	}
	
	NSMutableArray *subtrees = [tree subtrees];
	NSRange dirty = dirtyIndexRange(subtrees, edit, newCharacters, length, changedRange);
	
	[subtrees removeObjectsInRange:dirty];
	dirty.length = 0;
	
	NSUInteger cursor = NSMaxRange([tree triggerRange]);
	if(dirty.location > 0)
	{
		cursor = NSMaxRange([[subtrees objectAtIndex:dirty.location-1] range]);
	}
	
	NSUInteger indexCursor = dirty.location;
	
	while(YES)
	{
		if(reuse(subtrees, cursor, indexCursor, changedRange))
		{
			[tree setRange:NSMakeRange([tree range].location, NSMaxRange([tree range]) + [edit changeInLength])];
			return ReparseSucceed;
		}
		
		ZParseTree *next = [self nextElementIn:newCharacters length:length cursor:&cursor];
		if([next parser] == _endParser)
		{
			[subtrees removeObjectsInRange:NSMakeRange(indexCursor, [subtrees count] - indexCursor)];
			setTreeEnd(tree, NSMaxRange([next range]), edit, changedRange);
			return ReparseSucceed;
		}
		else if(next != nil)
		{
			[subtrees insertObject:next atIndex:indexCursor];
			indexCursor++;
			ZExtendRange(changedRange, [next range]);
		}
	}	
}
@end

static NSRange dirtyIndexRange(NSArray *subtrees, ZStringEdit *edit, unichar *newCharacters, unsigned int length, NSRange *changedRange)
{
	NSRange dirtyIndexRange = NSMakeRange(0, 0);
	for(ZParseTree *sub in subtrees)
	{
		int result = [[sub parser] reparse:sub edit:edit newCharacters:newCharacters length:length changedRange:changedRange];
		if(result == ReparseSucceed)
		{
			dirtyIndexRange.location++;
		}
		else if(result == ReparseFailed)
		{
			dirtyIndexRange.length++;
		}
	}
	return dirtyIndexRange;
}

static BOOL reuse(NSMutableArray *subtrees, NSUInteger cursor, NSUInteger indexCursor, NSRange *changedRange)
{
	while(changedRange != NULL && cursor >= NSMaxRange(*changedRange) && [subtrees count] > indexCursor)
	{
		ZParseTree *first = [subtrees objectAtIndex:indexCursor];
		if(cursor <= [first range].location)
		{
			return YES;
		}
		else
		{
			ZExtendRange(changedRange, [first range]);
			[subtrees removeObject:first];
		}
	}
	return NO;
}

static void setTreeEnd(ZParseTree *tree, NSUInteger end, ZStringEdit *edit, NSRange *changedRange)
{
	int oldEnd = NSMaxRange([edit translatedRange:[tree range]]);
	[tree setRange:NSMakeRange([tree range].location, end - [tree range].location)];
	
	if(oldEnd != end)
	{
		ZExtendRange(changedRange, NSMakeRange(MAX(oldEnd, end), 0));
	}
}