//
//  ZSyntax.m
//  sEdit
//
//  Created by 増田 勇樹 on Thu Aug 05 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "ZSyntax.h"
#import "ZSyntax-Private.h"

#import "ZColoringParser.h"
#import "ZParseTree.h"
#import "ZParenthesisAction.h"
#import "ZStack.h"
#import "ZStringEdit.h"
#import "ZUnicharCache.h"
#import "ZMode.h"

NSString *ZSyntaxDescriptionFileName = @"syntax.plist";

/********************************************
* keys for dictionrary in syntax.plist file
********************************************/
NSString *ZSyntaxExtendsKey = @"extends";
NSString *ZSyntaxTopStateKey = @"topState";

/*****************************
* Keys for Colors Dictionary 
******************************/
NSString *ColorsTextKey = @"text";
NSString *ColorsBackgroundKey = @"background";
NSString *ColorsCommentKey = @"comment";
NSString *ColorsPreprocessorKey = @"preprocessor";
NSString *ColorsStringKey = @"string";
NSString *ColorsParenthesisKey = @"parenthesis";
NSString *ColorsKeywordKey = @"keyword";
NSString *ColorsCharacterKey = @"character";
NSString *ColorsNumberKey = @"number";

NSString *ActionContextColorsKey = @"colors";
NSString *ActionContextStringKey = @"string";

NSDictionary *extended_syntax_desc(NSDictionary *desc);

@implementation ZSyntax
+ (ZSyntax *)syntaxWithDescription:(NSDictionary *)description
{
	return [[[ZSyntax alloc] initWithDescription:description] autorelease];
}

- (id)initWithDescription:(NSDictionary *)desc
{
	self = [super init];
	if(self)
	{		
		ZColoringParser *coloringParser = [ZColoringParser parserWithDescription:[desc objectForKey:ZSyntaxTopStateKey] topLevel:YES];
		_coloringParseTree = [[ZParseTree treeWithParser:coloringParser] retain];
	}
	return self;
}

- (void)dealloc
{
	[_cachedUnichars release];
	[_coloringParseTree release];
	[super dealloc];
}

- (void)applyToTextStorage:(NSTextStorage *)storage
		editedRange:(NSRange)editedRange
		changeInLength:(NSInteger)changeInLength
		colors:(NSDictionary *)colors
{
	NSRange editedInOld = NSMakeRange(editedRange.location, editedRange.length - changeInLength);
	ZStringEdit *edit = [ZStringEdit editInRange:editedInOld
								   changeInLength:changeInLength];
	if(_cachedUnichars == nil)
	{
		_cachedUnichars = [[ZUnicharCache cache] retain];
	}
	
	[_cachedUnichars edit:editedInOld changeInLength:changeInLength newString:[storage string]];
	
	[[_coloringParseTree parser] reparse:_coloringParseTree
								 edit:edit
								 newCharacters:[_cachedUnichars unichars]
								 length:[_cachedUnichars length]
								 changedRange:&editedRange];
	[self applyColors:colors textStorage:storage range:editedRange];
}

- (void)applyColors:(NSDictionary *)colors textStorage:(NSTextStorage *)storage range:(NSRange)range
{
	[storage beginEditing];
	NSDictionary *args = [NSDictionary dictionaryWithObjectsAndKeys:colors, ActionContextColorsKey,
									   storage, ActionContextStringKey, nil];
	[_coloringParseTree runActionInRange:range context:args];
	[storage endEditing];
}

- (void)showMatchIn:(NSTextStorage *)storage
		editedRange:(NSRange)editedRange
		changeInLength:(NSInteger)changeInLength
		selectedTextAttributes:(NSDictionary *)selected
{
	if(changeInLength <= 0)
	{
		return;
	}
	NSRange range;
	id att = [storage attribute:parenthesisAttributeName atIndex:editedRange.location effectiveRange:&range];
	ZParenthesisAction *a = [(ParenthesisAttribute *)att action];
	
	if(a != nil && [a isEnd])
	{
		ZStack *stack = [ZStack stack];
		[stack push:a];
		int loc = range.location - 1;
		while(loc >= 0)
		{
			att = [storage attribute:parenthesisAttributeName atIndex:loc effectiveRange:&range];
			a = [(ParenthesisAttribute *)att action];
			if(a != nil)
			{
				if(![a isEnd])
				{
					if([a correspond] == [stack peep])
					{
						[stack pop];
						if([stack count] == 0)
						{
							[self highlightRange:range inStorage:storage selectedTextAttributes:selected];
							return;
						}
					}
					else
					{
						NSBeep();
						return;
					}
				}
				else
				{
					[stack push:a];
				}
			}
			loc = range.location - 1;
		}
		NSBeep();
	}
}
@end
